package outils;

import com.thoughtworks.xstream.XStream;
import com.thoughtworks.xstream.io.xml.DomDriver;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.List;
import modeles.Cours;
import modeles.Etudiant;
import modeles.Horaire;
import modeles.ListeCours;
import modeles.ResultatInscription;
import modeles.Seance;
import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

public class Deserialisation {

    public static Horaire construireHoraire(String xmlHoraire) throws Exception {
        SAXBuilder builder = new SAXBuilder(false);
        Document doc;
        try {
            //Source: http://developerstips.wordpress.com/2008/06/12/jdom-readparse-xml-from-string/
            StringReader lecteurString = new StringReader(xmlHoraire);
            doc = builder.build(lecteurString);
            if (doc.getRootElement().getName().equals("erreur")) {
                throw new Exception(doc.getRootElement().getChild("message").getText());
            } else {
                XStream xstream = new XStream(new DomDriver());
                xstream.alias("horaire", Horaire.class);
                xstream.alias("seance", Seance.class);
                Horaire horaireEtudiant = (Horaire) xstream.fromXML(xmlHoraire);
                return horaireEtudiant;
            }
        } catch (JDOMException e) {
            System.out.println("JDOMException dans ControleurHoraireEtudiant(): " + e);
            return null;
        } catch (IOException e) {
            System.out.println("IOException dans ControleurHoraireEtudiant(): " + e);
            return null;
        }
    }
    
    public static ListeCours construireListeCours(String xmlListeCours) {
        XStream xstream = new XStream(new DomDriver());
        xstream.alias("coursDisponibles", ArrayList.class);
        xstream.alias("listeCours", List.class);
        xstream.alias("etudiant", Etudiant.class);
        xstream.alias("seance", Seance.class);
        xstream.alias("cours", Cours.class);
        List<Cours> listeCours = (List<Cours>) xstream.fromXML(xmlListeCours);
        return new ListeCours(listeCours);
    }
    
    public static ResultatInscription construireResultatInscription(String xmlResultatInscription) {
        XStream xstream = new XStream(new DomDriver());
        xstream.alias("resultat", ResultatInscription.class);
        ResultatInscription resultatInscription = (ResultatInscription) xstream.fromXML(xmlResultatInscription);
        return resultatInscription;
    }
    
}
