package outils;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.Multimap;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import modeles.Cours;
import modeles.Etudiant;
import modeles.Seance;
import org.jdom.*;
import org.jdom.input.*;

public class Deserialisation {

    private String nomFichierEtudiants, nomFichierHoraires, nomFichierInscriptions;

    public void setNomFichierEtudiants(String nomFichierEtudiants) {
        this.nomFichierEtudiants = nomFichierEtudiants;
    }

    public void setNomFichierHoraires(String nomFichierHoraires) {
        this.nomFichierHoraires = nomFichierHoraires;
    }

    public void setNomFichierInscriptions(String nomFichierInscriptions) {
        this.nomFichierInscriptions = nomFichierInscriptions;
    }

    public Map<String, Cours> lireCours() {
        Map mapCours = new HashMap<String, Cours>();
        try {
            File fichier = new File(nomFichierHoraires);
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(fichier);
            Element racine = doc.getRootElement();
            List listeNoeudsCours = racine.getChildren("cours");
            Iterator iCours = listeNoeudsCours.iterator();
            Multimap mapInscritsCours = lireCoursInscrits("sigleCours");
            while (iCours.hasNext()) {
                Cours cours = new Cours();
                Element elementCours = (Element) iCours.next();
                String sigleCours = elementCours.getAttributeValue("sigle");
                cours.setSigle(sigleCours);
                int capacite = Integer.parseInt(elementCours.getChild("capacite").getText());
                cours.setCapacite(capacite);
                try {
                    String textePrealables = elementCours.getChild("prealables").getText();
                    String[] siglesPrealables = textePrealables.split(",");
                    for (String siglePrealable : siglesPrealables) {
                        Cours coursPrealable = new Cours();
                        coursPrealable.setSigle(siglePrealable);
                        cours.addPrealable(coursPrealable);
                    }
                } catch (NullPointerException e) {
                    // Cours sans prealable
                }
                for (Object idEtudiant : mapInscritsCours.get(sigleCours)) {
                    Etudiant etudiant = new Etudiant();
                    etudiant.setId((String) idEtudiant);
                    cours.addEtudiantInscrit(etudiant);
                }
                try {
                    List<Element> noeudsSeance = elementCours.getChildren("seance");
                    for (int iNoeudsSeance = 0; iNoeudsSeance < noeudsSeance.size(); iNoeudsSeance++) {
                        Element e = noeudsSeance.get(iNoeudsSeance);
                        String textSeance = e.getText();
                        Seance seance = new Seance();
                        seance.setTexte(textSeance);
                        cours.addSeance(seance);
                    }
                } catch (NullPointerException e) {
                    // Cours sans seance
                    System.out.println("Cours "+cours.getSigle()+" sans seance");
                }
                try {
                    List<Element> noeudsSeance = elementCours.getChildren("laboratoire");
                    for (int iNoeudsSeance = 0; iNoeudsSeance < noeudsSeance.size(); iNoeudsSeance++) {
                        Element e = noeudsSeance.get(iNoeudsSeance);
                        String textSeance = e.getText();
                        Seance seance = new Seance();
                        seance.setTexte(textSeance);
                        seance.setLaboratoire(true);
                        cours.addSeance(seance);
                    }
                } catch (NullPointerException e) {
                    // Cours sans laboratoire
                }
                mapCours.put(cours.getSigle(), cours);
            }
        } catch (IOException e) {
            System.out.println("IOException dans Deserialisation->lireCours(): " + e);
        } catch (JDOMException e) {
            System.out.println("JDOMException dans Deserialisation->lireCours(): " + e);
        }
        return mapCours;
    }

    public Map<String, Etudiant> lireEtudiants() {
        Map mapEtudiants = new HashMap<String, Etudiant>();
        try {
            File fichier = new File(nomFichierEtudiants);
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(fichier);
            Element racine = doc.getRootElement();
            List listeNoeudsEtudiant = racine.getChildren("etudiant");
            Iterator iEtudiant = listeNoeudsEtudiant.iterator();
            Multimap mapInscritsCours = lireCoursInscrits("idEtudiant");
            while (iEtudiant.hasNext()) {
                Element elemenEtudiant = (Element) iEtudiant.next();
                String idEtudiant = elemenEtudiant.getAttributeValue("id");
                String texteSiglesSuivis = elemenEtudiant.getChild("suivis").getText();
                String[] siglesSuivis = texteSiglesSuivis.split(",");
                Etudiant etudiant = new Etudiant();
                etudiant.setId(idEtudiant);
                for (String sigleSuivi : siglesSuivis) {
                    Cours cours = new Cours();
                    cours.setSigle(sigleSuivi);
                    etudiant.addCoursSuivi(cours);
                }
                for (Object sigleCoursInscrit : mapInscritsCours.get(idEtudiant)) {
                    Cours cours = new Cours();
                    cours.setSigle((String) sigleCoursInscrit);
                    etudiant.addCoursInscrit(cours);
                }
                mapEtudiants.put(idEtudiant, etudiant);
            }
        } catch (IOException e) {
            System.out.println("IOException dans Deserialisation->lireEtudiants(): " + e);
        } catch (JDOMException e) {
            System.out.println("JDOMException dans Deserialisation->lireEtudiants(): " + e);
        }
        return mapEtudiants;
    }

    private Multimap<String, String> lireCoursInscrits(String nomKey) {
        Multimap mapInscritsCours = ArrayListMultimap.create();
        try {
            File fichier = new File(nomFichierInscriptions);
            SAXBuilder builder = new SAXBuilder(false);
            Document doc = builder.build(fichier);
            Element racine = doc.getRootElement();
            List listeNoeudsCours = racine.getChildren("cours");
            racine.getDescendants();
            Iterator iCours = listeNoeudsCours.iterator();
            while (iCours.hasNext()) {
                Element elementCours = (Element) iCours.next();
                String sigleCours = elementCours.getAttributeValue("sigle");
                List listeNoeudsInscription = elementCours.getChildren("inscrit");
                Iterator iInscrits = listeNoeudsInscription.iterator();
                while (iInscrits.hasNext()) {
                    Element elementInscrit = (Element) iInscrits.next();
                    String idEtudiant = elementInscrit.getText();
                    if (nomKey.equals("idEtudiant")) {
                        mapInscritsCours.put(idEtudiant, sigleCours);
                    } else {
                        mapInscritsCours.put(sigleCours, idEtudiant);
                    }
                }
            }
        } catch (IOException e) {
            System.out.println("IOException dans Deserialisation->lireCoursInscrits(): " + e);
        } catch (JDOMException e) {
            System.out.println("JDOMException dans Deserialisation->lireCoursInscrits(): " + e);
        }
        return mapInscritsCours;
    }
}
