package outils;

import modeles.ResultatValidation;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import modeles.Cours;
import modeles.Etudiant;
import modeles.Seance;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;
import org.jdom.xpath.XPath;

// Suit le patron de conception Singleton
public class PersistenceDonees {

    private static PersistenceDonees persistenceDonees;
    private Deserialisation serialisation;
    private static final String nomFichierEtudiants = "etudiants.xml";
    private static final String nomFichierHoraires = "horaires.xml";
    private static final String nomFichierInscriptions = "inscriptions.xml";

    private PersistenceDonees() {
        serialisation = new Deserialisation();
        serialisation.setNomFichierEtudiants(nomFichierEtudiants);
        serialisation.setNomFichierHoraires(nomFichierHoraires);
        serialisation.setNomFichierInscriptions(nomFichierInscriptions);
    }

    public static PersistenceDonees getInstance() {
        if (persistenceDonees != null) {
            return persistenceDonees;
        }
        return new PersistenceDonees();
    }

    public Cours cours(String idCours) {
        Map<String, Cours> mapCours = serialisation.lireCours();
        return mapCours.get(idCours);
    }

    public Etudiant etudiant(String idEtudiant) {
        Map<String, Etudiant> mapEtudiants = serialisation.lireEtudiants();
        return mapEtudiants.get(idEtudiant);
    }

    public Map<String, Cours> coursTrimestreCourant() {
        return serialisation.lireCours();
    }

    public List<Seance> horaireEtudiant(String idEtudiant) {
        Map<String, Cours> mapCours = serialisation.lireCours();
        Map<String, Etudiant> mapEtudiants = serialisation.lireEtudiants();
        Etudiant etudiant = mapEtudiants.get(idEtudiant);
        List listeSeances = new ArrayList<Seance>();
        for (Cours coursInscrit : etudiant.getCoursInscrits()) {
            Cours coursDisponible = mapCours.get(coursInscrit.getSigle());
            for (Seance seance : coursDisponible.getSeances()) {
                seance.setCours(coursInscrit);
                listeSeances.add(seance);
            }
        }
        return listeSeances;
    }

    public ResultatValidation inscrireCours(String idEtudiant, String sigleCours) {
        ValidationInscription validateur = new ValidationInscription(idEtudiant, sigleCours);
        ResultatValidation resultatValidation = validateur.validerInscription();
        if(!resultatValidation.isValide())
            return resultatValidation;
        //Source: http://www.mkyong.com/java/how-to-modify-xml-file-in-java-jdom/
        //Source: http://notetodogself.blogspot.com/2007/11/parse-xml-with-jdom-and-xpath.html
        try {
            SAXBuilder builder = new SAXBuilder();
            File xmlFile = new File(nomFichierInscriptions);
            Document doc = (Document) builder.build(xmlFile);            
            XPath xpath = XPath.newInstance("//cours[@sigle=\""+sigleCours+"\"]");
            List listeCoursModifier = xpath.selectNodes(doc);
            Element elementCours = null;
            try{
                elementCours = (Element) listeCoursModifier.get(0);
            } catch(java.lang.IndexOutOfBoundsException e) {
                resultatValidation.setMessage("Cours non existant");
                resultatValidation.setValide(false);
                return resultatValidation;
            }
            Element elementInscrit = new Element("inscrit").setText(idEtudiant);
            elementCours.addContent(elementInscrit);
            XMLOutputter xmlOutput = new XMLOutputter();
            xmlOutput.setFormat(Format.getPrettyFormat());
            xmlOutput.output(doc, new FileWriter(nomFichierInscriptions));
            return resultatValidation;
        } catch (IOException e) {
            resultatValidation.setMessage("Erreur dans la lecture/ecriture des fichiers: " + e);
            resultatValidation.setValide(false);
            return resultatValidation;
        } catch (JDOMException e) {
            resultatValidation.setMessage("Erreur dans le parsing du document: " + e);
            resultatValidation.setValide(false);
            return resultatValidation;        }
    }
        
    public Map<String, Etudiant> etudiants() {
        return serialisation.lireEtudiants();
    }
}
