package outils;

import modeles.ResultatValidation;
import java.util.List;
import modeles.Cours;
import modeles.Etudiant;

public class ValidationInscription {
    private PersistenceDonees pd;
    private String idEtudiant;
    private String sigleCours;

    public ValidationInscription(String idEtudiant, String sigleCours) {
        this.idEtudiant = idEtudiant;
        this.sigleCours = sigleCours;
        pd = PersistenceDonees.getInstance();
    }

    public ResultatValidation validerInscription() {
        ResultatValidation resultat = new ResultatValidation(false);
        if(validerStatusEtudiant() == false) {
            resultat.setMessage("L'étudiant n'est pas inscrit cet trimestre");
            return resultat;
        }
        if(validerDisponibilite() == false) {
            resultat.setMessage("Le cours n'a plus places disponibiles");
            return resultat;
        }
        if(validerDejaInscrit() == false) {
            resultat.setMessage("L'étudiant a déjà inscrit le cours");
            return resultat;
        }
        if(validerPrealables() == false) {
            resultat.setMessage("L'étudiant n'a pas suivi un cours préalable requis");
            return resultat;
        }
        return new ResultatValidation(true);
    }
   
    
    private boolean validerDejaInscrit() {
        Etudiant etudiant = pd.etudiant(idEtudiant);
        if (etudiant.aInscrit(sigleCours) == false) {
            return true;
        }
        return false;
    }
    
    private boolean validerDisponibilite() {
        Cours cours = pd.cours(sigleCours);
        if(cours.getDisponibilite() < 1)
            return false;
        return true;
    }
    
    private boolean validerPrealables() {
        Etudiant etudiant = pd.etudiant(idEtudiant);
        Cours cours = pd.cours(sigleCours);
        List<Cours> listeCoursPrealables = cours.getPrealables();
        for(Cours coursPrealable : listeCoursPrealables) {
            if(etudiant.aSuivi(coursPrealable.getSigle()) == false)
                return false;
        }
        return true;
    }

    private boolean validerStatusEtudiant() {
        try {
            Etudiant etudiant = pd.etudiant(idEtudiant);
            if(etudiant == null)
                return false;
            return true;
        } catch(NullPointerException e) {
            return false;
        }
    }
    
}
